;;;--------------------------------------------------------------------------
;;;  INFORMATION:
;;;--------------------------------------------------------------------------
;;;  
;;;  This model simulates the experiment described in Van Rij, J., Van Rijn, 
;;;  H., & Hendriks, P. (submitted). Cognitive Architectures and Language 
;;;  Acquisition: A Case Study in Pronoun Comprehension.
;;; 
;;;  author:    Jacolien van Rij
;;;  date:      April 2008
;;;  address:	Center for Language and Cognition Groningen
;;;  			University of Groningen
;;;				P.O.Box 716, 9700AS Groningen
;;;				J.C.van.Rij@rug.nl
;;;
;;;  This simulates the performance of a group of child participants on 
;;;  sentences with normal and slowed-down speech rate.
;;;  
;;;--------------------------------------------------------------------------
;;;  TO DO:
;;;--------------------------------------------------------------------------
;;;
;;;  The model is implemented in ACT-R 6, but the old utility-function (P-CG)
;;;  must be used for this model: 
;;;     * replace utility-and-reward-1.lisp in modules\ 
;;;       with the file utility-and-reward.lisp in extras\old-utility\
;;;
;;;  Use (do-experiment n) to run n simulations.
;;;  The normal speech rate can be changed with *d-time-n* (the mean time in
;;;  seconds between different words). The slow speech rate can be changed 
;;;  *d-time-s*. Normal Dutch speech rate is about 4 syllables per second.
;;;  Because in the experimental sentences words contain on average 1.28 
;;;  syllables per word, the time between words is around 310 ms per word.
;;;  In addition, the yes-bias can be changed with *yes-bias*.
;;; 
;;;--------------------------------------------------------------------------

(defvar *d-time-n* .31)
(defvar *d-time-s* .49)
(defvar *d-sd* .01)
;; overige globale variabelen
(defvar *response* nil)
(defvar *verbose* nil)
(defvar *%Refl* 10)
(defvar *yes-bias* 80)
(defvar *case* 0)
(defvar *training* nil)
(defvar *condition* -1)

;; zinnen met gegevens normal speech rate conditie
(defvar *sentences-N* '( 
    (:nr 1 :ans false :img coreferential :class pronoun 
    :sentence ("de" "aap" "bijt" "hem" "in" "een" "poot"))
    (:nr 2 :ans false :img disjoint :class reflexive
    :sentence ("de" "aap" "bijt" "zichzelf" "in" "een" "poot"))
    (:nr 3 :ans true :img disjoint :class pronoun 
    :sentence ("de" "aap" "kietelt" "hem" "met" "een" "veer"))
    (:nr 4 :ans true :img coreferential :class reflexive
    :sentence ("de" "aap" "kietelt" "zichzelf" "met" "een" "veer"))
    (:nr 5 :ans false :img coreferential :class pronoun 
    :sentence ("het" "konijn" "schminkt" "hem" "met" "een" "krijtje"))
    (:nr 6 :ans false :img disjoint :class reflexive
    :sentence ("het" "konijn" "schminkt" "zichzelf" "met" "een" "krijtje"))
    (:nr 7 :ans true :img disjoint :class pronoun 
    :sentence ("de" "krokodil" "wijst" "naar" "hem" "met" "een" "vinger"))
    (:nr 8 :ans true :img coreferential :class reflexive
    :sentence ("de" "krokodil" "wijst" "naar" "zichzelf" "met" "een" "vinger"))
    (:nr 9 :ans false :img coreferential :class pronoun 
    :sentence ("de" "olifant" "slaat" "hem" "met" "een" "hamer"))
    (:nr 10 :ans false :img disjoint :class reflexive 
    :sentence ("de" "olifant" "slaat" "zichzelf" "met" "een" "hamer"))
    (:nr 11 :ans true :img disjoint :class pronoun 
    :sentence ("de" "hond" "bindt" "hem" "vast" "aan" "een" "boom"))
    (:nr 12 :ans true :img coreferential :class reflexive 
    :sentence ("de" "hond" "bindt" "zichzelf" "vast" "aan" "een" "boom"))
    (:nr 13 :ans false :img coreferential :class pronoun 
    :sentence ("de" "schildpad" "ziet" "hem" "op" "de" "televisie"))
    (:nr 14 :ans false :img disjoint :class reflexive
    :sentence ("de" "schildpad" "ziet" "zichzelf" "op" "de" "televisie"))
    (:nr 15 :ans true :img disjoint :class pronoun 
    :sentence ("de" "kat" "schildert" "hem" "met" "een" "kwast"))
    (:nr 16 :ans true :img coreferential :class reflexive
    :sentence ("de" "kat" "schildert" "zichzelf" "met" "een" "kwast"))
))

;; zinnen met gegevens slow speech rate conditie
(defvar *sentences-S* '( 
	(:nr 1 :ans false :img coreferential :class pronoun 
    :sentence ("het" "konijn" "bijt" "hem" "in" "een" "oor"))
	(:nr 2 :ans false :img disjoint :class reflexive 
    :sentence ("het" "konijn" "bijt" "zichzelf" "in" "een" "oor"))
	(:nr 3 :ans true :img disjoint :class pronoun 
    :sentence ("de" "beer" "kietelt" "hem" "met" "een" "grassprietje"))
	(:nr 4 :ans true :img coreferential :class reflexive 
    :sentence ("de" "beer" "kietelt" "zichzelf" "met" "een" "grassprietje"))
	(:nr 5 :ans false :img coreferential :class pronoun 
    :sentence ("de" "olifant" "schminkt" "hem" "met" "een" "kwast"))
	(:nr 6 :ans false :img disjoint :class reflexive 
    :sentence ("de" "olifant" "schminkt" "zichzelf" "met" "een" "kwast"))
	(:nr 7 :ans true :img disjoint :class pronoun 
    :sentence ("de" "kat" "wijst" "naar" "hem" "met" "een" "poot"))
	(:nr 8 :ans true :img coreferential :class reflexive 
    :sentence ("de" "kat" "wijst" "naar" "zichzelf" "met" "een" "poot"))
	(:nr 9 :ans false :img coreferential :class pronoun 
    :sentence ("de" "pingun" "slaat" "hem" "met" "een" "pan"))
	(:nr 10 :ans false :img disjoint :class reflexive 
    :sentence ("de" "pingun" "slaat" "zichzelf" "met" "een" "pan"))
	(:nr 11 :ans true :img disjoint :class pronoun 
    :sentence ("de" "egel" "bindt" "hem" "vast" "aan" "een" "lantaarnpaal"))
	(:nr 12 :ans true :img coreferential :class reflexive 
    :sentence ("de" "egel" "bindt" "zichzelf" "vast" "aan" "een" "lantaarnpaal"))
	(:nr 13 :ans false :img coreferential :class pronoun 
    :sentence ("het" "schaap" "ziet" "hem" "op" "de" "computer"))
	(:nr 14 :ans false :img disjoint :class reflexive 
    :sentence ("het" "schaap" "ziet" "zichzelf" "op" "de" "computer"))
	(:nr 15 :ans true :img disjoint :class pronoun 
    :sentence ("het" "varken" "tekent" "hem" "met" "een" "potlood"))
	(:nr 16 :ans true :img coreferential :class reflexive 
    :sentence ("het" "varken" "tekent" "zichzelf" "met" "een" "potlood"))
	))


(defvar *matchN* (map 'vector #' (lambda (x) (if (equal (getf x :ans) 'false) 0 (if (equal (getf x :ans) 'true) 1 nil))) *sentences-N*))
(defvar *classN* (map 'vector #' (lambda (x) (if (equal (getf x :class) 'reflexive) 0 (if (equal (getf x :class) 'pronoun) 1 nil))) *sentences-N*))
(defvar *typesN* (map 'vector #' (lambda (x y) (+ x (* 2 y))) *classN* *matchN*))

(defvar *matchS* (map 'vector #' (lambda (x) (if (equal (getf x :ans) 'false) 0 (if (equal (getf x :ans) 'true) 1 nil))) *sentences-S*))
(defvar *classS* (map 'vector #' (lambda (x) (if (equal (getf x :class) 'reflexive) 0 (if (equal (getf x :class) 'pronoun) 1 nil))) *sentences-S*))
(defvar *typesS* (map 'vector #' (lambda (x y) (+ x (* 2 y))) *classS* *matchS*))

; Initialization outputfile
(defvar *training-file* "trainingfile.csv")
(with-open-file
    (poutfile-training *training-file*
                :if-exists :append
                :if-does-not-exist :create
                :direction :io)

    (write-string
        (format nil "~A~%"
            "case;class;score;certainty;bi-OT" )
    poutfile-training)
)

(defvar *data-file* "datafile.csv")
(with-open-file
    (poutfile *data-file*
                :if-exists :append
                :if-does-not-exist :create
                :direction :io)

    (write-string
        (format nil "~A~%"
            "case;training;condition;nr;class;img;answer;score" )
    poutfile)
)

;; ((item (nth (random (list-length '("hem" "zichzelf"))) '("hem" "zichzelf")))
;; n trainingsitem, vr aanroepen eerst (pdisable-fct '(answer_correct answer_incorrect answer_unsure))
(defun one-training (&optional (p 50))
  
    (let* 
        ((item (if (< (act-r-random 100) p) (nth 1 '("hem" "zichzelf")) (nth 0 '("hem" "zichzelf"))))
           (type (if (equal item "hem") 'disjoint 'coreferential)) 
           (runtime (+ *d-time-n* (act-r-noise *d-sd*)))
           )
        (reset-declarative-finsts)
        (add-new-goal-to-DM 'practice type item 't)
        (format t "." )
        (format *verbose* "~tpresenting: ~10S >" item)
        ;;(format t "~,3f~t~10S: " (mp-time) item)
        (while (<= runtime 0) (setf runtime (+ *d-time-n* (act-r-noise *d-sd*))))
        (run runtime)
        (mod-chunk-fct (no-output (goal-focus)) (list 'time nil))
        (run 2)
        (format *verbose* "~toutput:~t~S~%" (chunk-slot-value-fct (no-output (goal-focus)) 'output))
        ;;(format t "~t~10S" (chunk-slot-value-fct (no-output (goal-focus)) 'output))
        ;;(format t "~t(~S)~%" (chunk-slot-value-fct (no-output (goal-focus)) 'certainty))
        (print-training)
      ) 
)

;; 'oefenen' in optimization
(defun training (times uni)

  (sgp :epl t)
  (penable-fct (all-productions))
  (pdisable-fct '(answer_correct answer_incorrect answer_unsure answer_sure no_answer function1))
  (penable-fct '(time-limit-reached time-limit-reached-retrieval start-practice output_nil))
  (format *verbose* "~%***~%UNIDIRECTIONAL OPTIMIZATION~%***~%" )
  (format t "~%train unidirectional")

  (dotimes (i uni)
    (format *verbose* "~4d)" i)
    (one-training *%Refl*)
  )

  (penable-fct '(function1))
  (format *verbose* "~%***~%BIDIRECTIONAL OPTIMIZATION~%***~%" )
  (format t "~%train bidirectional")
  
  (dotimes (i (- times uni))
    (format *verbose* "~4d)" (+ i uni))
    (one-training *%Refl*)
  )

  (sgp :epl nil)
  (penable-fct (all-productions))
  (pdisable-fct '(time-limit-reached time-limit-reached-retrieval start-practice output_nil))
)



;; complete trial, zonder oefening
(defun do-trial (list-item time-difference time-deviation)

  (sgp :epl nil)
  (penable-fct (all-productions))
  (pdisable-fct '(time-limit-reached time-limit-reached-retrieval start-practice output_nil)) 
  (reset-declarative-finsts)

  (let* 
      ((item (getf list-item :nr))
       (correct-ans (getf list-item :ans)) 
       (image (getf list-item :img))
       (old-time (+ (mp-time) 3))
       (maxtime 0)
       (window (open-exp-window "Comprehension of pronouns" :visible nil)))

    (format *verbose* "~%~t~2d NEW TRIAL~%" item)
    (format *verbose* "~tSENTENCE:        : \"~{~a~^ ~}\"~%" (getf list-item :sentence))
    (format t "~3d: " item)

    ;; schedule words
    (dolist (word (getf list-item :sentence))
      (let*
          ((new-time 0)
           (temp 0))

        (while (>= old-time temp)
            (setf new-time (+ time-difference (act-r-noise time-deviation)))
            (setf temp (+ new-time old-time)))
        ;;(format *verbose* "~t~S: ~,3f => run ~,3f sec~%" word old-time new-time)
        (setf old-time temp)
        (schedule-event old-time 'new-word-sound :params (list word) :priority :max :details word)
        (setq maxtime (+ old-time 10.0))
    ))

    ;; process sentence words
    (add-new-goal-to-DM 'start image nil)
    (run maxtime)
    (mod-chunk-fct (no-output (goal-focus)) (list 'state 'start-response 'time 'done))
    (penable-fct '(answer_unsure answer_sure answer_correct answer_incorrect no_answer))

    ;; create window for handling answer
    (install-device window)
    (add-text-to-exp-window :text "collect answer" :x 50 :y 160)

    ;; generate answer
    (let ((top-goal (no-output (goal-focus))))
      (eval `(goal-focus ,top-goal))
      (run 5)
    )

    (close-exp-window)

    ;; process response
    (format *verbose* "~tCORRECT ANSWER   : ~S~%" correct-ans)
    (format *verbose* "~tANSWER           : ~S~%" *response*)
    (format t "~S," *response*)
    
    
    
    ;; return value
    (if (equal *response* "x") (return-from do-trial (list *response* nil))
      (if (or 
           (and (equal *response* "t") (equal correct-ans 'true))
           (and (equal *response* "f") (equal correct-ans 'false)))
          (return-from do-trial (list *response* 1))
          (return-from do-trial (list *response* 0)))))
)

(defun experiment ()

  (format *verbose* "~%***~%NORMAL SPEECH RATE~%***~%" )
  (format t "~%normal speech rate" )

  ;; normal speech rate
  (let*
      ((sentence-list (permute-list *sentences-N*))
       (data nil)
       (score (make-array (length *sentences-N*) :initial-element nil))
       (test-order nil)
       (*condition* 0))

    (dolist (sentence sentence-list)
      (setf *response* nil)
      (setf data (do-trial sentence *d-time-n* *d-sd*))
      (if (equal (car data) "x") (setf data (do-trial sentence  *d-time-n* *d-sd*)))
      (print-trial sentence data)
      (setf (elt score (1- (getf sentence :nr))) (second data))
      (push (getf sentence :nr) test-order)
      )
    ;;(format *verbose* "~%~tRESULTS:         : ~a~%"  *score*)
  )

  (format *verbose* "~%***~%SLOW SPEECH RATE~%***~%" )
  (format t "~%slow speech rate" )

  ;; slow speech rate
  (let*
      ((sentence-list (permute-list *sentences-S*))
       (data nil)
       (score (make-array (length *sentences-S*) :initial-element nil))
       (test-order nil)
       (*condition* 1))

    (dolist (sentence sentence-list)
      (setf *response* nil)
      (setf data (do-trial sentence *d-time-s* *d-sd*))
      (if (equal (car data) "x") (setf data (do-trial sentence  *d-time-s* *d-sd*)))
      (print-trial sentence data)
      (setf (elt score (1- (getf sentence :nr))) (second data))
      (push (getf sentence :nr) test-order)
      )

  )
)

(defun do-experiment ()
  (reset)
  (penable-fct  (all-productions))

  ;; oefenen:
  (setf *training* -75)
  (format t "~%Experiment...~%")
  (experiment)
  
  (format t "~%~d - 0. Training: 50 unidirectional optimization " *case*)
  (training 50 50)
  (setf *training* -25)
  (format t "~%Experiment...~%")
  (experiment)

  (format t "~%~d - 1. Training: 25 unidirectional + 25 bidirectional optimization " *case*)
  (training 50 25)
  (setf *training* 25)
  (format t "~%Experiment...~%")
  (experiment)

  ;; oefenen:
  (format t "~%~d - 2. Training: 50 bidirectional optimization " *case*)
  (training 50 0)
  (setf *training* 75)
  (format t "~%Experiment...~%")
  (experiment)

  (format t "~%~d - 3. Training: 50 bidirectional optimization " *case*)
  (training 50 0)
  (setf *training* 125)
  (format t "~%Experiment...~%")
  (experiment)

  (format t "~%~d - 4. Training: 50 bidirectional optimization " *case*)
  (training 50 0)
  (setf *training* 175)
  (format t "Experiment...~%")
  (experiment)

  (format t "~%~d - 5. Training: 50 bidirectional optimization " *case*)
  (training 50 0)
  (setf *training* 225)
  (format t "Experiment...~%")
  (experiment)

  (format t "~%~d - 6. Training: 50 bidirectional optimization " *case*)
  (training 50 0)
  (setf *training* 275)
  (format t "Experiment...~%")
  (experiment)

  (format t "~%~d - 7. Training: 50 bidirectional optimization " *case*)
  (training 50 0)
  (setf *training* 325)
  (format t "Experiment...~%")
  (experiment)

  (format t "~%~d - 8. Training: 50 bidirectional optimization " *case*)
  (training 50 0)
  (setf *training* 375)
  (format t "Experiment...~%")
  (experiment)


  (format t "~%~d - 9. Training: 50 bidirectional optimization " *case*)
  (training 50 0)
  (setf *training* 425)
  (format t "Experiment...~%")
  (experiment)

  (format t "~%~d - 10. Training: 50 bidirectional optimization " *case*)
  (training 50 0)
  (setf *training* 475)
  (format t "Experiment...~%")
  (experiment)

  (format t "~%~d - 11. Training: 50 bidirectional optimization " *case*)
  (training 100 0)
  (setf *training* 575)
  (format t "Experiment...~%")
  (experiment)
)

(defun repeat-experiment (n)
  (reset)
  (dotimes (i n)
    (setf *case* i)
    (setf *training* nil)
    (do-experiment)
    )
)

(defun experiment-case(c)
  (reset)
  (setf *case* c)
  (setf *training* nil)
  (do-experiment)

)

(defmethod rpm-window-key-event-handler ((win rpm-window) key)
  (if (or (string= key "f") (string= key "t")(string= key "x"))
    (setf *response* (string key))))

(defun add-new-goal-to-DM (state image inp &optional (tt nil))
    (let* ((top-goal (gentemp "process-sentence")))
        (progn
            (eval `(add-dm
                (,top-goal ISA process-sentence state ,state time ,tt image ,image input ,inp certainty nil)))
            (eval `(goal-focus ,top-goal))
            top-goal)))


(defun print-trial (info-sentence info-response)
    (with-open-file
    	(poutfile *data-file*
    			    :if-exists :append
    			    :if-does-not-exist :create
    			    :direction :io)   
    ;; Print casenr, training, condition
    (write-string (format nil "~d;" *case*) poutfile)
    (write-string (format nil "~d;" *training*) poutfile)
    (write-string (format nil "~d;" *condition*) poutfile)   
    ;; print nr,class, img
    (write-string (format nil "~d;" (getf info-sentence :nr)) poutfile)
    (write-string (format nil "~s;" (getf info-sentence :class)) poutfile)
    (write-string (format nil "~s;"  (getf info-sentence :img)) poutfile)    
    ;;print answer,score
    (write-string (format nil "~s;" (first info-response)) poutfile)
    (write-string (format nil "~d~%" (second info-response)) poutfile)
    )
)


(defun print-training ()
    (with-open-file
    	(poutfile-training *training-file*
    			    :if-exists :append
    			    :if-does-not-exist :create
    			    :direction :io)
    ;; Print casenr
    (write-string (format nil "~d;" *case*) poutfile-training)
    (write-string (format nil "~S;" (chunk-slot-value-fct (no-output (goal-focus)) 'syntax)) poutfile-training)
    (write-string (format nil "~d;" 
                          (if (or (and (equal (chunk-slot-value-fct (no-output (goal-focus)) 'syntax) 'pronoun)
                                       (equal (chunk-slot-value-fct (no-output (goal-focus)) 'output) 'disjoint))
                                  (and (equal (chunk-slot-value-fct (no-output (goal-focus)) 'syntax) 'reflexive)
                                       (equal (chunk-slot-value-fct (no-output (goal-focus)) 'output) 'coreferential)))
                              1 0)) poutfile-training)
    (write-string (format nil "~S;" (chunk-slot-value-fct (no-output (goal-focus)) 'certainty)) poutfile-training)
    (write-string (format nil "~S~%" (chunk-slot-value-fct (no-output (goal-focus)) 'biot)) poutfile-training))
)

;;-----------------------------------

(clear-all)

(define-model testing!

(set-audloc-default :onset lowest :attended nil)

;; Parameters settings
    (sgp :v nil :trace-detail low :needs-mouse nil :show-focus t
         :esc t :epl t :pct nil
        :pl t :ut -100 :egs 1 
        :rt -100 :bll 0.5 :mas 2 :ans 0.3 
        :declarative-finst-span 3 :ncnar nil
        :act nil :save-buffer-trace t)

  
(chunk-type process-sentence state time process image syntax input cand1 cand2 optimal output certainty biot)
(chunk-type word spelling) ; lexical items
(chunk-type syntax-element type cue) ; syntax element 
(chunk-type key letter)
(chunk-type candidate cand input)
(chunk-type constraint input ranking violates)

(add-dm 
 ;; states
 (start isa chunk) 
 (attend isa chunk)
 (lex-access isa chunk) 
 (syntax-retrieve isa chunk)
 (respond isa chunk)
 (done isa chunk)
 (get-candidate isa chunk)
 (get-cand2 isa chunk)
 (get-optimal isa chunk)
 (unsolved isa chunk)        
 (next-candidate isa chunk)
 (forced-output isa chunk)
 (done-uni isa chunk)
 (function isa chunk)
 (start-response isa chunk)
 (start-production isa chunk)
 (practice isa chunk)
         (done_bi_pos isa chunk)
        (done_bi_neg isa chunk)     (time-limit-reached isa chunk)
 ;;concepts
        (interpretation isa chunk)  (production isa chunk)
        (first isa chunk) (second isa chunk) (third isa chunk)
        (pronoun isa chunk)         (reflexive isa chunk)
        (coreferential isa chunk)   (disjoint isa chunk)

        ;; Candidates - production
        (pro-co-prod isa candidate cand pronoun input coreferential)
        (pro-dis-prod isa candidate cand pronoun input disjoint)
        (self-co-prod isa candidate cand reflexive input coreferential)
        (self-dis-prod isa candidate cand reflexive input disjoint)
        ;; Candidates - interpretation
        (pro-co-int isa candidate cand coreferential input pronoun)
        (pro-dis-int isa candidate cand disjoint input pronoun)
        (self-co-int isa candidate cand coreferential input reflexive)
        (self-dis-int isa candidate cand disjoint input reflexive)
        ;; Dummy candidate
        (no-candidate isa candidate cand nil input nil)


        ;; Constraint - Principle A 
        (principle_A_refl_int isa constraint input reflexive ranking first violates disjoint)
        (principle_A_pro_int isa constraint input pronoun ranking first violates nil)  
        (principle_A_dis_prod isa constraint input disjoint ranking first violates reflexive)
        (principle_A_co_prod isa constraint input coreferential ranking first violates nil)
        ;; Constraint - Refential Economy 
        (avoidPronouns_refl_int isa constraint input reflexive ranking second violates nil)
        (avoidPronouns_pro_int isa constraint input pronoun ranking second violates nil)
        (avoidpronouns_co_prod isa constraint input coreferential ranking second violates pronoun)
        (avoidPronouns_dis_prod isa constraint input disjoint ranking second violates pronoun) 
        ;; Dummy constraint
        (no-constraint isa constraint input nil ranking third violates nil)
 
)

;; stop items in declaratief geheugen

(let* ((memory-list nil)
       (sentences (concatenate 'list *sentences-N* *sentences-S*)))
  (dolist (item sentences)
    (dolist (x (getf item :sentence))
      ;; alleen string-items en als ze nog niet staan in memory
      (if (not (member x memory-list :test #'string-equal))
        (let ((word (new-name "word")))
          (push x memory-list)
          ;; woorden in dm
          (eval `(add-dm (,word isa word spelling ,x)))
          (eval `(sdp ,word :creation-time -1e7 :references 1e5))
          ;; bijbehorende syntax in dm
          (if (not (or (equal x "hem") (equal x "zichzelf"))) 
              (let ((syntax-element (new-name "syntax-")))
                (eval `(add-dm (,syntax-element isa syntax-element cue ,x)))
                (eval `(sdp ,syntax-element :creation-time -1e4 :references 1e3))))
        ))
  ))
)

;; * build initial tree
(add-dm 
 (syntax-pronoun isa syntax-element type pronoun cue "hem")
 (syntax-reflexive isa syntax-element type reflexive cue "zichzelf"))


(sdp syntax-pronoun :creation-time -1000000 :references 75000) ; have to experiment with this
(sdp syntax-reflexive :creation-time -1000000 :references 25000) 

    ;; Set baselevel actiation, by defining creation time and number of references
    (sdp pronoun :creation-time -1000000 :reference-count 75000)
    (sdp reflexive :creation-time -1000000 :reference-count 25000)
    (sdp disjoint :creation-time -1000000 :reference-count 50000)
    (sdp coreferential :creation-time -1000000 :reference-count 50000)

    (sdp pro-dis-int :creation-time -50000 :reference-count 500)
    (sdp pro-co-int :creation-time -50000 :reference-count 500)
    (sdp self-dis-int :creation-time -50000 :reference-count 500)
    (sdp self-co-int :creation-time -50000 :reference-count 500)

    (sdp pro-dis-prod :creation-time -50000 :reference-count 500)
    (sdp pro-co-prod :creation-time -50000 :reference-count 500)
    (sdp self-dis-prod :creation-time -50000 :reference-count 500)
    (sdp self-co-prod :creation-time -50000 :reference-count 500)

    (sdp no-candidate :creation-time -50000 :reference-count 1000)

    (sdp principle_A_refl_int :creation-time -50000 :reference-count 750)
    (sdp principle_A_pro_int :creation-time -50000 :reference-count 1000)
    (sdp principle_A_dis_prod :creation-time -50000 :reference-count 850)
    (sdp principle_A_co_prod :creation-time -50000 :reference-count 850)

    (sdp avoidPronouns_refl_int :creation-time -50000 :reference-count 500)
    (sdp avoidPronouns_pro_int :creation-time -50000 :reference-count 750)
    (sdp avoidpronouns_co_prod :creation-time -50000 :reference-count 650)
    (sdp avoidPronouns_dis_prod :creation-time -50000 :reference-count 650)

    (sdp no-constraint :creation-time -50000 :reference-count 1000)

    ;;Sji: This is the strength of association from source j to chunk i. 
     (add-sji (pronoun syntax-pronoun 15)(syntax-pronoun pronoun 15))
     (add-sji (reflexive syntax-reflexive 15)(syntax-reflexive reflexive 15))

    (add-sji (pronoun principle_A_pro_int 15)(principle_A_pro_int pronoun 15))
    (add-sji (pronoun avoidPronouns_pro_int 10)(avoidPronouns_pro_int pronoun 10))
    (add-sji (reflexive principle_A_refl_int 7)(principle_A_refl_int reflexive 7))
    (add-sji (reflexive avoidPronouns_refl_int 5)(avoidPronouns_refl_int reflexive 5))

    (add-sji (disjoint principle_A_dis_prod 10)(principle_A_dis_prod disjoint 10))
    (add-sji (coreferential principle_A_co_prod 7)(principle_A_co_prod coreferential 7))
    (add-sji (disjoint avoidPronouns_dis_prod 10)(avoidPronouns_dis_prod disjoint 10))
    (add-sji (coreferential avoidPronouns_co_prod 5)(avoidPronouns_co_prod coreferential 5))

    (add-sji (pro-dis-int pronoun 3) (pronoun pro-dis-int 3))
    (add-sji (pro-dis-int disjoint 3) (disjoint pro-dis-int 3))
    (add-sji (pro-co-int pronoun 3) (pronoun pro-co-int 3))
    (add-sji (pro-co-int coreferential 3) (coreferential pro-co-int 3))
    (add-sji (self-dis-int reflexive 3) (reflexive self-dis-int 3))
    (add-sji (self-dis-int disjoint 3) (disjoint self-dis-int 3))
    (add-sji (self-co-int reflexive 3) (reflexive self-co-int 3))
    (add-sji (self-co-int coreferential 3) (coreferential self-co-int 3))

    (add-sji (pro-dis-int pronoun 3) (pronoun pro-dis-prod 3))
    (add-sji (pro-dis-prod disjoint 3) (disjoint pro-dis-prod 3))
    (add-sji (pro-co-prod pronoun 3) (pronoun pro-co-prod 3))
    (add-sji (pro-co-prod coreferential 3) (coreferential pro-co-prod 3))
    (add-sji (self-dis-prod reflexive 3) (reflexive self-dis-prod 3))
    (add-sji (self-dis-prod disjoint 3) (disjoint self-dis-prod 3))
    (add-sji (self-co-prod reflexive 3) (reflexive self-co-prod 3))
    (add-sji (self-co-prod coreferential 3) (coreferential self-co-prod 3))

;; productieregels: interpretatie van woorden

(P attend-aural
   =goal>
      ISA         process-sentence
      state       start
      time nil
   =aural-location>
     isa      audio-event
   ?aural>
      state    free   
==>
   =goal>
      state   attend
   +aural>
     isa      sound
     event    =aural-location
)


(P clear-retrieval
   =goal>
      ISA         process-sentence
      state       start
      time nil
   =aural-location>
     isa      audio-event
   ?aural>
      state    free  
   ?retrieval>
      buffer full
==>
   =goal>
      state   attend
   +aural>
     isa      sound
     event    =aural-location
   -retrieval>
)

(P aural-error
   =goal>
      ISA         process-sentence
      state       attend
      time nil
   ?aural>
     state error
==>
   =goal>
      state       start
)



(P lexical-access
   =goal>
      ISA         process-sentence
      state       attend
      time nil
   =aural>
     isa          sound
     content     =spelling
   ?retrieval>
     state free
     buffer empty
==>
   =goal>
      state       lex-access
  +retrieval>
     isa word
     spelling =spelling
     !output! (=spelling attended)
)


(p set-retrieval-cues
   =goal>
      ISA         process-sentence
      state       lex-access
      time nil
   =retrieval>
      isa         word
      spelling    =word
==>
   =goal>
     state        syntax-retrieve
   +retrieval>
     isa          syntax-element
     cue          =word
     
)

;; overige woorden
(p attach-syntax-element1
   =goal>
      ISA         process-sentence
      state       syntax-retrieve
      time nil
   =retrieval>
     isa          syntax-element
     type         nil
==>
   =goal>
     state start
)

;; nieuw ---------------------------------------------------------------
;; extra p voor starten oefenen optimalisatie
;; !!! misschien geen pc mogelijk maken?!!!
(P start-practice
   =goal>
      ISA         process-sentence
      state       practice
      image       =img
      input       =word
      time t
      !eval! t
==>
   =goal>
     state        syntax-retrieve
     input        nil
   +retrieval>
     isa          syntax-element
     cue          =word 
)

;; als er sprake is van pronouns of reflexives, start OT
;; set retrieval of first candidate
(p start-interpretation
   =goal>
      ISA         process-sentence
      state       syntax-retrieve
   =retrieval>
     isa          syntax-element
     type         =value
==>
   =goal>
     state        get-candidate
     time t
     syntax       =value
     input        =value
     process      interpretation
   +retrieval>
     isa	candidate
     input	=value
     :recently-retrieved	nil
)

;; If an utterance must be produced, start by retrieving a relevant candidate
(P start_prod
	=goal>
		isa			   process-sentence
        input		    =input
		image	    =image
		state			start-production
		process			production
		time			t		
==>
	=goal>
		state			get-candidate
	+retrieval>
		isa			    candidate
		input		    =input
)


;; If one candidate is being retrieved, retrieve a second candidate
(P cand1
	=goal>
		isa		process-sentence
		input		=input
        state			get-candidate
		process			=process
		time			t	
	=retrieval>
		isa 			candidate
		cand			=cand
==>
	=goal>
		cand1			=cand
		state			get-cand2
	+retrieval>
		isa			    candidate
		input		    =input
		- cand			=cand
		:recently-retrieved	nil
)


;;If two candidates are being retrieved, retrieve the first constraint
(P cand2
	=goal>
		isa		process-sentence
		input		=input
		cand1		=val
		state		get-cand2
		time		t	
	=retrieval>
		isa 		candidate
		cand		=cand
        !safe-eval! (not (equal =val =cand))
==>
	=goal>
        	cand2		=cand
		state		get-optimal
	+retrieval>
		isa		constraint
                input		    =input
                ranking		    first
)

(P new-cand
	=goal>
		isa		process-sentence
		input		=input
		cand1		=val
		state		get-cand2
		time		t	
	=retrieval>
		isa 		candidate
		cand		=cand
        !safe-eval! (equal =val =cand)
==>
	=goal>
		cand1			=cand
		state			get-cand2
	+retrieval>
		isa			    candidate
		input		    =input
		- cand			=cand
		:recently-retrieved	nil
)
(spp new-cand :failure t)


;; If the both retrieved candidates are the same, replace one
;;(P evaluation_same
;;	=goal>
;;		isa		process-sentence
;;		input		    =input
;;		cand1			=val1
;;		cand2			=val1 
;;		state			get-optimal
;;		time			t	
;;	=retrieval>
;;		isa 			constraint
;;		ranking			first
;;==>
;;	=goal>
;;		cand1			=val1
;;		cand2			nil 
;;		state			get-cand2
;;	+retrieval>
;;		isa			 candidate
;;		input		    =input
;;		- cand			=val1
;;)

;;(spp  evaluation_same :failure t)


;; If the first constraint does not violate one of the candidates, 
;; retrieve the second constraint
(P evaluation_constr1
	=goal>
		isa		process-sentence
		input		    =input
		cand1		    =val1
		cand2		    =val2
		state		    get-optimal
		time		    t	
	=retrieval>
		isa 		    constraint
		ranking		    first
        violates        nil 
==>
	=goal>
		state           get-optimal
	+retrieval>
		isa             constraint
		input           =input        
                ranking         second
)


;; If the second constraint does not violate one of the candidates, 
;; retrieve the third constraint 
(P evaluation_constr2
	=goal>
		isa		process-sentence
		input		    =input
		cand1			=val1
		cand2			=val2
		state			get-optimal
		time			t	
	=retrieval>
		isa 			constraint
		ranking			second
        violates        nil
==>
	=goal>
		state			unsolved
	+retrieval>
		isa			    constraint
                ranking		    third
)		


;; If the constraint violates the first candidate, 
;; mark the second candidate as optimal and retrieve the next candidate
(P evaluation_violates1
	=goal>
		isa		process-sentence
		input		    =input
		cand1			=violates
		cand2			=val2
		state			get-optimal
		process			=process        
		time			t	
	=retrieval>
		isa 			constraint
        violates        =violates
==>
	=goal>
		optimal			=val2
		state			next-candidate
                certainty               t
	+retrieval>
		isa			    candidate
        input           nil
		- cand			=val2
		- cand			=violates
)


;; If the constraint violates the second candidate, 
;; mark the first candidate as optimal and retrieve the next candidate
(P evaluation_violates2
	=goal>
		isa		process-sentence
		input		    =input
		cand1			=val1
		cand2			=violates
		state			get-optimal
        process			=process
		time			t	
	=retrieval>
		isa 			constraint
        violates        =violates
==>
	=goal>
		optimal			=val1
		state			next-candidate
                certainty               t
	+retrieval>
		isa			    candidate
        input           nil
		- cand			=val1
		- cand			=violates
)


;; If no candidate can be retrieved at all an output cannot be given
(P no_candidate1
	=goal>
		isa		process-sentence
		input		    =input
		state			get-candidate
                time t
	?retrieval>
		state			error
==>
	=goal>
		optimal 		nil
		state			forced-output
)


;; If a second candidate cannot be retrieved, 
;; mark the first candidate as being optimal. Note that this is a forced choice! 
(P no_candidate2
	=goal>
		isa		process-sentence
		cand1			=cand
		state			get-cand2
                time t
	?retrieval>
		state			error
==>
	=goal>
		optimal 		=cand
		state			done-uni
)

;; If a constraint cannot be retrieved, mark an arbitrary candidate as being optimal. 
;; This is also a forced choice situation! 
(P no_constr12
	=goal>
		isa		process-sentence
		cand1			=cand1
		cand2			=cand2
		state			get-optimal
                time t
	?retrieval>
		state			error
==>
	=goal>
		optimal			=cand1
		state			done-uni
)


;; If no next candidate can be retrieved, the unidirectional optimal interpretation is found 
(P no_next_candidate
	=goal>
		isa		process-sentence
		optimal			=optimal
		state			next-candidate
        process			interpretation
		time			t
	=retrieval>
		isa             candidate
        input           nil       
==>
	=goal>
		state			done-uni
)


;; If there are no constraints left to be retrieved, 
;; mark an arbitrary candidate as being optimal and retrieve a next candidate
(P no_next_constraint
	=goal>
		isa		process-sentence
		input		    =input
		cand1			=val1
		cand2			=val2
		state			unsolved
        process			=process
		time			t	
	=retrieval>
		isa             constraint
        input           nil
==>
	=goal>
		optimal			=val2
		state			next-candidate
	+retrieval>
		isa			    candidate
        input           nil
		- cand			=val1
		- cand			=val2
)


(P no_next_constraint2
	=goal>
		isa		process-sentence
		input		    =input
		cand1			=val1
		cand2			=val2
		state			unsolved
        process			=process
		time			t	
	=retrieval>
		isa             constraint
        input           nil
==>
	=goal>
		optimal			=val1
		state			next-candidate
	+retrieval>
		isa			    candidate
        input           nil
		- cand			=val1
		- cand			=val2
)

;; If no next candidate can be retrieved, the unidirectional optimal interpretation is found. 
;; Find the bidirectional interpretation
(P bidirectional
	=goal>
		isa		process-sentence
                input           =input 
		optimal		=optimal
		state		next-candidate
                process		interpretation
		time		t	
	=retrieval>
		isa             candidate
                input           nil   
==>
	=goal>
		state			 function
                output =optimal
                !output! (Finished unidirectional process. Optimal =optimal)
)

;; If the unidirectional optimal interpretation is found, 
;; Find the bidirectional optimal interpretation 
(P bidirectional2
	=goal>
	isa		process-sentence
        input           =input
	optimal			=optimal
	state			done-uni
        process			interpretation
	time			t
                

==>
	=goal>
	state           function
        output =optimal
        !output! (Finished unidirectional process. Optimal =optimal)
)


;; feedback reflexieven
(P finished_uni
	=goal>
		isa			   process-sentence
		optimal			=optimal
		state			next-candidate
        process			interpretation
                certainty               t
		time			t	
	=retrieval>
		isa             candidate
        input           nil   
==>
	=goal>
		input 		=optimal
        output          =optimal
		state			function
        !output! (Finished unidirectional process)
)

;; If you were interpreting, start producing
(P function1
	=goal>
		isa	process-sentence
        input           =input
		image		=image
		optimal			=optimal
		state			function
        process         interpretation
		time			t	
==>
	=goal>
		input		    =optimal
        state			start-production        
		process			production	
)


;; If no next candidate can be retrieved and the output is equal 
;; to the original input, than the interpretation is correct
(P output_bi_pos
	=goal>
		isa			    process-sentence		
        syntax	=val
		optimal			=val
		state			next-candidate
        process 		production
		time			t	
	=retrieval>
		isa             candidate
        input           nil 
==>
	=goal>
		state			done_bi_pos
                certainty               t
                biot t
                time nil
                
)
                                                                        
;; If no next candidate can be retrieved and the output is not equal 
;; to the original input, than the interpretation is not correct
;; !!! extra productie? !!!
(P output_bi_neg
	=goal>
		isa			    process-sentence
        optimal			=val
	  - syntax	=val
		state			next-candidate
        process 		production
		time			t	
	=retrieval>
		isa             candidate
        input           nil 
==>
	=goal>
		state			done_bi_neg
                certainty               nil
                biot t
                time   nil

)   

;; If the timelimit is reached, stop processing 
(P time-limit-reached
	=goal>
		isa 			process-sentence
		time			nil
==>
	=goal>
		state			time-limit-reached
		time			time-limit-reached
)

(P  time-limit-reached-retrieval
	=goal>
		isa 			process-sentence   
		time			nil
        ?retrieval>
                buffer full
==>
	-retrieval>
	=goal>
		state			time-limit-reached
		time			time-limit-reached
)

(P remove-retrieval
	=goal>
		isa 			process-sentence 
		state			time-limit-reached  
		time			time-limit-reached
        ?retrieval>
                buffer full
==>
	-retrieval>
)

(P output_nil
	=goal>
       isa	process-sentence
        output  nil
        optimal =optimal
        state                   time-limit-reached
	time			time-limit-reached
==>
	=goal>
        output  =optimal
)

;; --------------------------------------------


(P stop-optimization1
   =goal>
      ISA         process-sentence
      time t
      ;;certainty nil
   =aural-location>
     isa      audio-event
   ?aural>
      state    free
      buffer empty   
   ?retrieval>
      state free
      buffer empty
==>
   =goal>
      state   attend
      time nil
   +aural>
     isa      sound
     event    =aural-location
)


(P stop-optimization2
   =goal>
      ISA         process-sentence
      time t
      ;;certainty t
      output =output
   ?retrieval>
      buffer full
   =aural-location>
     isa      audio-event
   ?aural>
      state    free
      buffer empty   
==>
   =goal>
      state   attend
      time nil
   -retrieval>
   +aural>
     isa      sound
     event    =aural-location
)

;; If answer is ALMOST found with high degree of certainty, mark optimal as output 
(P answer_sure
	=goal>
		isa 			process-sentence
                state start-response
		time			done
                optimal                 =optimal
                output			nil
                certainty               t
==>
   =goal>
     output       =optimal
)

;; if an answer has to be given, then compare the optimal candidate with the picture
;; If the timelimit is reached, stop processing 
(P answer_correct
	=goal>
		isa 			process-sentence
                state start-response
		time			done
                image                   =same
                output			=same
                certainty               t
   ?imaginal>
     state free
==>
   =goal>
     state        respond
    ;; time        nil
   +imaginal>
      isa         key
      letter      "t"
)

;; If the timelimit is reached, stop processing 
(P answer_incorrect
	=goal>
		isa 			process-sentence
                state start-response
		time			done
                image                   =image
                output			=optimal
                - output               =image
                certainty               t
   ?imaginal>
     state free
==>
   =goal>
     state        respond
    ;; time        nil
   +imaginal>
      isa         key
      letter      "f"
)



;; chance performance + yes-bias 
(P answer_unsure
	=goal>
		isa 			process-sentence
                state                 start-response
		time			done     
                optimal                 =optimal
                certainty               nil
   ?imaginal>
     state free
==>
   !safe-bind! =letter (if (> *yes-bias* (act-r-random 100)) "t" "f")
   =goal>
     state        respond
   +imaginal>
      isa         key
      letter      =letter
)

;; when not pronoun/reflexive is not processed
(P no_answer
	=goal>
		isa 			process-sentence
                state                 start-response
		time			done     
                optimal                 nil
                output                  nil
                certainty               nil
   ?imaginal>
     state free
==>
   =goal>
     state        respond
   +imaginal>
      isa         key
      letter      "x"
)

;; productieregels: genereer output
(P respond
   =goal>
      ISA         process-sentence
      state       respond
      time done
   =imaginal>
      isa         key
      letter      =letter
   ?manual>   
      state       free
==>
   =goal>
      state       done
   +manual>
      ISA         press-key
      key         =letter
)

;; if the anaphor is not heard, the sentence can be played again
;; nog te maken

;; FEEDBACK

;; interpretation for reflexive found
 (spp finished_uni :success t)
;; Success in bidirectional optimization
(spp output_bi_pos :success t)  
;; Failure in bidirectional optimization
(spp output_bi_neg :failure t)    


(setf *actr-enabled-p* t)
)
